package net.cattaka.rdbassistant.jspf.core.bean;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

public class TableInfo {
	public DatabaseMetaData databaseMetaData;
	public String name;
	private DatabaseInfo parentInfo;
	private List<ColumnInfo> columnInfoList = null;
	
	public TableInfo(DatabaseInfo parentInfo, DatabaseMetaData databaseMetaData, String name) {
		this.parentInfo = parentInfo;
		this.databaseMetaData = databaseMetaData;
		this.name = name;
	}
	
	private void createColumnInfoList() throws SQLException {
		Connection connection = databaseMetaData.getConnection();
		ResultSet rs = null;
		HashSet<String> primaryKeys = new HashSet<String>(); 
		
		// 主キーの取得
		try {
			rs = databaseMetaData.getPrimaryKeys(connection.getCatalog(), parentInfo.getName(), this.name);
			while(rs.next()) {
				String columnName = "";
				columnName = rs.getString("COLUMN_NAME");
				primaryKeys.add(columnName);
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
		}
		
		// 列一覧の取得
		try {
			rs = databaseMetaData.getColumns(connection.getCatalog(), parentInfo.getName(), this.name, "%");
			this.columnInfoList = new ArrayList<ColumnInfo>();
			
			while(rs.next()) {
				String columnName = "";
				boolean primaryKey = false;
				
				columnName = rs.getString("COLUMN_NAME");
				primaryKey = primaryKeys.contains(columnName);
				
				ColumnInfo columnInfo = new ColumnInfo(databaseMetaData, columnName);
				columnInfo.setPrimaryKey(primaryKey);
				this.columnInfoList.add(columnInfo);
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
		}
	}

	public ColumnInfo getColumnInfo(String columnName) throws SQLException {
		ColumnInfo result = null;
		List<ColumnInfo> cil = getColumnInfoList();
		for (ColumnInfo ci : cil) {
			if (ci.getName().equals(columnName)) {
				result = ci;
				break;
			}
		}
		return result;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public List<ColumnInfo> getColumnInfoList() throws SQLException {
		if (columnInfoList == null) {
			this.createColumnInfoList();
		}
		return columnInfoList;
	}
}
