package net.cattaka.rdbassistant.oracle;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.cattaka.rdbassistant.core.SqlEditorSelection;
import net.cattaka.rdbassistant.gui.table.ResultSetTableModel;
import net.cattaka.rdbassistant.gui.table.StaticResultSetTableModel;
import net.cattaka.util.ExceptionHandler;

public class OracleSqlEditorSelection implements SqlEditorSelection {
	public static String SELECT_DATABASE_LIST = "SELECT USERNAME FROM ALL_USERS ORDER BY USERNAME";
	public static String SELECT_TABLES_LIST = "SELECT A.TABLE_NAME,B.COMMENTS,A.NUM_ROWS,A.TABLESPACE_NAME,A.CLUSTER_NAME,A.IOT_NAME,A.PCT_FREE,A.PCT_USED,A.INI_TRANS,A.MAX_TRANS,A.INITIAL_EXTENT,A.NEXT_EXTENT,A.MIN_EXTENTS,A.MAX_EXTENTS,A.PCT_INCREASE,A.FREELISTS,A.FREELIST_GROUPS,A.LOGGING,A.BACKED_UP,A.BLOCKS,A.EMPTY_BLOCKS,A.AVG_SPACE,A.CHAIN_CNT,A.AVG_ROW_LEN,A.AVG_SPACE_FREELIST_BLOCKS,A.NUM_FREELIST_BLOCKS,A.DEGREE,A.INSTANCES,A.CACHE,A.TABLE_LOCK,A.SAMPLE_SIZE,A.LAST_ANALYZED,A.PARTITIONED,A.IOT_TYPE,A.TEMPORARY,A.SECONDARY,A.NESTED,A.BUFFER_POOL,A.ROW_MOVEMENT,A.GLOBAL_STATS,A.USER_STATS,A.DURATION,A.SKIP_CORRUPT,A.MONITORING FROM ALL_TABLES A, ALL_TAB_COMMENTS B WHERE A.OWNER = B.OWNER AND A.TABLE_NAME=B.TABLE_NAME AND A.OWNER=? ORDER BY TABLE_NAME";
	public static String SELECT_VIEWS_LIST = "SELECT A.VIEW_NAME,A.TEXT_LENGTH,A.TEXT,A.TYPE_TEXT_LENGTH,A.TYPE_TEXT,A.OID_TEXT_LENGTH,A.OID_TEXT,A.VIEW_TYPE_OWNER,A.VIEW_TYPE,A.SUPERVIEW_NAME FROM ALL_VIEWS A, ALL_TAB_COMMENTS B WHERE A.OWNER = B.OWNER AND A.VIEW_NAME=B.TABLE_NAME AND A.OWNER=? ORDER BY VIEW_NAME";
	public static String SELECT_SEQUENCES_LIST = "SELECT SEQUENCE_NAME,MIN_VALUE,MAX_VALUE,INCREMENT_BY,CYCLE_FLAG,ORDER_FLAG,CACHE_SIZE,LAST_NUMBER FROM ALL_SEQUENCES WHERE SEQUENCE_OWNER=? ORDER BY SEQUENCE_NAME";
	public static String[] SELECT_DETAIL_NAME_LIST = {
		"COLUMNS",
		"CONSTRAINTS"
	};
	public static String[] SELECT_DETAIL_SQL_LIST = {
		"SELECT A.COLUMN_NAME,B.COMMENTS,A.DATA_TYPE,A.DATA_PRECISION,A.DATA_SCALE,A.DATA_LENGTH,A.NULLABLE,A.DATA_DEFAULT,A.DATA_TYPE_MOD,A.DATA_TYPE_OWNER,A.COLUMN_ID,A.DEFAULT_LENGTH,A.NUM_DISTINCT,A.LOW_VALUE,A.HIGH_VALUE,A.DENSITY,A.NUM_NULLS,A.NUM_BUCKETS,A.LAST_ANALYZED,A.SAMPLE_SIZE,A.CHARACTER_SET_NAME,A.CHAR_COL_DECL_LENGTH,A.GLOBAL_STATS,A.USER_STATS,A.AVG_COL_LEN,A.CHAR_LENGTH,CHAR_USED FROM ALL_TAB_COLUMNS A, ALL_COL_COMMENTS B WHERE A.OWNER = B.OWNER AND A.TABLE_NAME = B.TABLE_NAME AND A.COLUMN_NAME = B.COLUMN_NAME AND A.OWNER=? AND A.TABLE_NAME=? ORDER BY COLUMN_ID",
		"SELECT AC.CONSTRAINT_NAME, AC.CONSTRAINT_TYPE, ACC.COLUMN_NAME, ACC.POSITION, AC.SEARCH_CONDITION, AC.R_OWNER, AC.R_CONSTRAINT_NAME, AC.DELETE_RULE, AC.STATUS, AC.DEFERRABLE, AC.DEFERRED, AC.VALIDATED, AC.GENERATED, AC.BAD, AC.RELY, AC.LAST_CHANGE, AC.INDEX_OWNER, AC.INDEX_NAME, AC.INVALID, AC.VIEW_RELATED FROM ALL_CONSTRAINTS AC LEFT OUTER JOIN ALL_CONS_COLUMNS ACC ON AC.OWNER=ACC.OWNER AND AC.TABLE_NAME=ACC.TABLE_NAME AND AC.CONSTRAINT_NAME=ACC.CONSTRAINT_NAME WHERE AC.OWNER=? AND AC.TABLE_NAME=? ORDER BY ACC.POSITION, AC.CONSTRAINT_NAME"
		//"SELECT CONSTRAINT_NAME,CONSTRAINT_TYPE,SEARCH_CONDITION,R_OWNER,R_CONSTRAINT_NAME,DELETE_RULE,STATUS,DEFERRABLE,DEFERRED,VALIDATED,GENERATED,BAD,RELY,LAST_CHANGE,INDEX_OWNER,INDEX_NAME FROM ALL_CONSTRAINTS WHERE OWNER=? AND TABLE_NAME=? ORDER BY CONSTRAINT_NAME"
	};
	
	private OracleRdbaConnection rdbConnection;
	
	public OracleSqlEditorSelection(OracleRdbaConnection rdbConnection) {
		this.rdbConnection = rdbConnection;
	}

	public String getDefaultDatabase() {
		return rdbConnection.getDefaultDatabase();
	}

	public List<String> getDatabaseList() {
		String sqlStr = SELECT_DATABASE_LIST;
		ArrayList<String> result = new ArrayList<String>();
		try {
			PreparedStatement stmt = rdbConnection.getConnection().prepareStatement(sqlStr);
			ResultSet rs = stmt.executeQuery();
			while(rs.next()) {
				result.add(rs.getString(1));
			}
			stmt.close();
		} catch (SQLException e) {
			ExceptionHandler.warn(e);
		}
		return result;
	}

	public List<String> getObjectTypeList() {
		ArrayList<String> result = new ArrayList<String>();
		result.add("TABLE");
		result.add("VIEW");
		result.add("SEQUENCE");
		return result;
	}
	
	public ResultSetTableModel getTableList(String database, String objectType) {
		String sqlStr = null;
		if (objectType.equals("TABLE")) {
			sqlStr = SELECT_TABLES_LIST;
		} else if (objectType.equals("VIEW")) {
			sqlStr = SELECT_VIEWS_LIST;
		} else if (objectType.equals("SEQUENCE")) {
				sqlStr = SELECT_SEQUENCES_LIST;
		}
		StaticResultSetTableModel result = new StaticResultSetTableModel();
		if (sqlStr != null) {
			PreparedStatement stmt = null;
			ResultSet rs = null;
			try {
				stmt = rdbConnection.getConnection().prepareStatement(sqlStr);
				stmt.setString(1, database);
				rs = stmt.executeQuery();
				result.extractResultSetData(rdbConnection, rs);
			} catch (SQLException e) {
				ExceptionHandler.warn(e);
			} finally {
				try {
					if (rs != null) {
						rs.close();
					}
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				}
				try {
					if (stmt != null) {
						stmt.close();
					}
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				}
			}
		}
		return result;
	}

	public List<ResultSetTableModel> getTableProperties(String database,String objectType, String table) {
		ArrayList<ResultSetTableModel> result = new ArrayList<ResultSetTableModel>();
		if (objectType.equals("TABLE") || objectType.equals("VIEW")) {
			for (int i=0;i<SELECT_DETAIL_NAME_LIST.length;i++) {
				PreparedStatement stmt = null;
				ResultSet rs = null;
				String sqlStr = String.format(SELECT_DETAIL_SQL_LIST[i], database, table);
				try {
					StaticResultSetTableModel pt = new StaticResultSetTableModel();
					// 実テーブルの列情報
					stmt = rdbConnection.getConnection().prepareStatement(sqlStr);
					stmt.setString(1, database);
					stmt.setString(2, table);
					rs = stmt.executeQuery();
					
					pt.extractResultSetData(rdbConnection, rs);
					pt.setTableName(SELECT_DETAIL_NAME_LIST[i]);
					
					result.add(pt);
				} catch (SQLException e) {
					ExceptionHandler.warn(e);
				} finally {
					try {
						if (rs != null) {
							rs.close();
						}
					} catch (SQLException e) {
						ExceptionHandler.warn(e);
					}
					try {
						if (stmt != null) {
							stmt.close();
						}
					} catch (SQLException e) {
						ExceptionHandler.warn(e);
					}
				}
			}
		} else if (objectType.equals("SEQUENCE")) {
			// なし
		}
		return result;
	}
		
	public String[] getColumnDefNameAsArray() {
		// TODO : ローカライズ
		return new String[]{"COLUMN_NAME", "TYPE", "NULLABLE", "PRIMARY KEY", "DEFUALT_VALUE", "COMMENT"};
	}
}
